class SubscriptionDeployerTemplate:
    """Create a service that can be used to be integrated with the core deployer"""

    @classmethod
    def name(cls, deployer, subscription):
        return f"{deployer['name']}-deployer-{subscription}-subscription"

    @classmethod
    def get(cls, subscription, subscription_list, deployer, config):
        cloud = deployer.get("cloud", None)
        if cloud == "AWS":
            return cls.aws(subscription, subscription_list, deployer, config)
        raise NotImplementedError("Unknown cloud")

    @classmethod
    def aws(cls, subscription, subscription_list, deployer, config):
        return [
            {
                "aws": {
                    "region": config.get("aws", "region", "ap-southeast-1"),
                    "account-id": config.get("aws", "account", None),
                    "deployment-role": config.get("aws", "deployment_role", ""),
                },
                "location": "./templates/aws/deployer-subscription.yaml",
                "template": {
                    "name": cls.name(deployer, subscription),
                    "parameters": {
                        "deployer-name": deployer["name"],
                        "subscription-name": subscription,
                    },
                    "subscriptions": subscription_list,
                },
                "functions": [],
            }
        ]



class ProjectTemplate:
    @classmethod
    def name(cls, deployer, project):
        return f"{deployer['name']}-deployer-project-{project}"

    @classmethod
    def get(cls, project, branches, deployer, config):
        cloud = deployer.get("cloud", None)
        if cloud == "AWS":
            return cls.aws(project, branches, deployer, config)
        raise NotImplementedError("Unknown cloud")

    @classmethod
    def aws(cls, project, branches, deployer, config):
        return [
            {
                "aws": {
                    "region": config.get("aws", "region", "ap-southeast-1"),
                    "account-id": config.get("aws", "account", None),
                    "deployment-role": config.get("aws", "deployment_role", ""),
                },
                "location": "./templates/aws/deployer-project.yaml",
                "template": {
                    "name": cls.name(deployer, project),
                    "parameters": {
                        "project": project,
                        "deployer": deployer["name"],
                        "branches": branches,
                        "service-trigger": f"{deployer['service-trigger']}",
                        "deployer-service-role": f"{deployer['service-role']}",
                    },
                },
                "functions": [],
            }
        ]



class CoreDeployerTemplate:
    """Create the core functionalities of the deployer with the workflow deployer"""

    @classmethod
    def name(cls, deployer):
        return f"{deployer['name']}-deployer-core-stack"

    @classmethod
    def get(cls, deployer, config):
        cloud = deployer.get("cloud", None)
        if cloud == "AWS":
            return cls.aws(deployer, config)
        raise NotImplementedError("Unknown cloud")

    @classmethod
    def aws(cls, deployer, config):
        return [
            {
                "aws": {
                    "region": config.get("aws", "region", "ap-southeast-1"),
                    "account-id": config.get("aws", "account", None),
                    "deployment-role": config.get("aws", "deployment_role", ""),
                },
                "location": "./templates/aws/deployer-core.yaml",
                "template": {
                    "name": cls.name(deployer),
                    "parameters": {
                        "stack-prefix": deployer["name"],
                        "registry-table": deployer["registry"],
                        "artifact-bucket": deployer["artifact"],
                        "deployment-workflow": f"{deployer['name']}-deployer-core",
                    },
                    "lambda-code-key": f"packages/{deployer['name']}-deployer-core-stack/",
                },
                "functions": [
                    {
                        "name": "package",
                        "location": "./core/",
                        "template-attribute": "lambda-code-key",
                        "bucket": deployer["artifact"],
                    }
                ],
            }
        ]
